@@ -18,9 +18,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
+use futures::{channel::oneshot, prelude::*};
 use libp2p_core::{muxing, upgrade, Transport};
 use libp2p_tcp::TcpConfig;
-use futures::{channel::oneshot, prelude::*};
 use std::sync::Arc;
 
 #[test]
@@ -32,14 +32,16 @@ fn client_to_server_outbound() {
     let bg_thread = async_std::task::spawn(async move {
         let mplex = libp2p_mplex::MplexConfig::new();
 
-        let transport = TcpConfig::new().and_then(move |c, e|
-            upgrade::apply(c, mplex, e, upgrade::Version::V1));
+        let transport = TcpConfig::new()
+            .and_then(move |c, e| upgrade::apply(c, mplex, e, upgrade::Version::V1));
 
         let mut listener = transport
             .listen_on("/ip4/127.0.0.1/tcp/0".parse().unwrap())
             .unwrap();
 
-        let addr = listener.next().await
+        let addr = listener
+            .next()
+            .await
             .expect("some event")
             .expect("no error")
             .into_new_address()
@@ -48,12 +50,19 @@ fn client_to_server_outbound() {
         tx.send(addr).unwrap();
 
         let client = listener
-            .next().await
+            .next()
+            .await
             .unwrap()
             .unwrap()
-            .into_upgrade().unwrap().0.await.unwrap();
+            .into_upgrade()
+            .unwrap()
+            .0
+            .await
+            .unwrap();
 
-        let mut outbound = muxing::outbound_from_ref_and_wrap(Arc::new(client)).await.unwrap();
+        let mut outbound = muxing::outbound_from_ref_and_wrap(Arc::new(client))
+            .await
+            .unwrap();
 
         let mut buf = Vec::new();
         outbound.read_to_end(&mut buf).await.unwrap();
@@ -62,13 +71,16 @@ fn client_to_server_outbound() {
 
     async_std::task::block_on(async {
         let mplex = libp2p_mplex::MplexConfig::new();
-        let transport = TcpConfig::new().and_then(move |c, e|
-            upgrade::apply(c, mplex, e, upgrade::Version::V1));
+        let transport = TcpConfig::new()
+            .and_then(move |c, e| upgrade::apply(c, mplex, e, upgrade::Version::V1));
 
         let client = Arc::new(transport.dial(rx.await.unwrap()).unwrap().await.unwrap());
         let mut inbound = loop {
-            if let Some(s) = muxing::event_from_ref_and_wrap(client.clone()).await.unwrap()
-                .into_inbound_substream() {
+            if let Some(s) = muxing::event_from_ref_and_wrap(client.clone())
+                .await
+                .unwrap()
+                .into_inbound_substream()
+            {
                 break s;
             }
         };
@@ -88,14 +100,16 @@ fn client_to_server_inbound() {
     let bg_thread = async_std::task::spawn(async move {
         let mplex = libp2p_mplex::MplexConfig::new();
 
-        let transport = TcpConfig::new().and_then(move |c, e|
-            upgrade::apply(c, mplex, e, upgrade::Version::V1));
+        let transport = TcpConfig::new()
+            .and_then(move |c, e| upgrade::apply(c, mplex, e, upgrade::Version::V1));
 
         let mut listener = transport
             .listen_on("/ip4/127.0.0.1/tcp/0".parse().unwrap())
             .unwrap();
 
-        let addr = listener.next().await
+        let addr = listener
+            .next()
+            .await
             .expect("some event")
             .expect("no error")
             .into_new_address()
@@ -103,15 +117,25 @@ fn client_to_server_inbound() {
 
         tx.send(addr).unwrap();
 
-        let client = Arc::new(listener
-            .next().await
-            .unwrap()
-            .unwrap()
-            .into_upgrade().unwrap().0.await.unwrap());
+        let client = Arc::new(
+            listener
+                .next()
+                .await
+                .unwrap()
+                .unwrap()
+                .into_upgrade()
+                .unwrap()
+                .0
+                .await
+                .unwrap(),
+        );
 
         let mut inbound = loop {
-            if let Some(s) = muxing::event_from_ref_and_wrap(client.clone()).await.unwrap()
-                .into_inbound_substream() {
+            if let Some(s) = muxing::event_from_ref_and_wrap(client.clone())
+                .await
+                .unwrap()
+                .into_inbound_substream()
+            {
                 break s;
             }
         };
@@ -123,14 +147,75 @@ fn client_to_server_inbound() {
 
     async_std::task::block_on(async {
         let mplex = libp2p_mplex::MplexConfig::new();
-        let transport = TcpConfig::new().and_then(move |c, e|
-            upgrade::apply(c, mplex, e, upgrade::Version::V1));
+        let transport = TcpConfig::new()
+            .and_then(move |c, e| upgrade::apply(c, mplex, e, upgrade::Version::V1));
 
         let client = transport.dial(rx.await.unwrap()).unwrap().await.unwrap();
-        let mut outbound = muxing::outbound_from_ref_and_wrap(Arc::new(client)).await.unwrap();
+        let mut outbound = muxing::outbound_from_ref_and_wrap(Arc::new(client))
+            .await
+            .unwrap();
         outbound.write_all(b"hello world").await.unwrap();
         outbound.close().await.unwrap();
 
         bg_thread.await;
     });
 }
+
+#[test]
+fn protocol_not_match() {
+    let (tx, rx) = oneshot::channel();
+
+    let _bg_thread = async_std::task::spawn(async move {
+        let mplex = libp2p_mplex::MplexConfig::new();
+
+        let transport = TcpConfig::new()
+            .and_then(move |c, e| upgrade::apply(c, mplex, e, upgrade::Version::V1));
+
+        let mut listener = transport
+            .listen_on("/ip4/127.0.0.1/tcp/0".parse().unwrap())
+            .unwrap();
+
+        let addr = listener
+            .next()
+            .await
+            .expect("some event")
+            .expect("no error")
+            .into_new_address()
+            .expect("listen address");
+
+        tx.send(addr).unwrap();
+
+        let client = listener
+            .next()
+            .await
+            .unwrap()
+            .unwrap()
+            .into_upgrade()
+            .unwrap()
+            .0
+            .await
+            .unwrap();
+
+        let mut outbound = muxing::outbound_from_ref_and_wrap(Arc::new(client))
+            .await
+            .unwrap();
+
+        let mut buf = Vec::new();
+        outbound.read_to_end(&mut buf).await.unwrap();
+        assert_eq!(buf, b"hello world");
+    });
+
+    async_std::task::block_on(async {
+        // Make sure they do not connect when protocols do not match
+        let mut mplex = libp2p_mplex::MplexConfig::new();
+        mplex.set_protocol_name(b"/mplextest/1.0.0");
+        let transport = TcpConfig::new()
+            .and_then(move |c, e| upgrade::apply(c, mplex, e, upgrade::Version::V1));
+        match transport.dial(rx.await.unwrap()).unwrap().await {
+            Ok(_) => {
+                assert!(false, "Dialing should fail here as protocols do not match")
+            }
+            _ => {}
+        }
+    });
+}
